/* Low-level screen operations for 320x400, 256-color mode */

/*
 Written (320x200 mode) by Bernie Roehl, November 1994 for VictorMaxx Technologies
 Updated to 320x400 mode by Freddie Feldman, VictorMaxx Technologies, Inc., August 1995
 Redistribute at will, but charge nothing and give credit where due
*/

#include "3d.h"

#include <stdlib.h>
#include <dos.h>
#include <mem.h>

extern void DisplayClear(void);
extern void Set320x400Mode(void);
extern void putp(void);


/*
 The screen can be thought of as being 320 by 400, but in this case
 it's treated as 640 by 200 with two screens side-by-side, each 320
 by 200.
*/

int DisplayWidth = 320, DisplayHeight = 200;  // size of each screen

static unsigned char far *screen;  // points to a malloc'd screen buffer

// row_starts[n] contains the offset into the screen of the start of row n:
static int row_starts[200];

static int oldmode = 0;    // original display mode

/*
 x_offset is the offset to left edge of the right-eye screen; it's set to
 half the width of the physical screen, so if it's 640 wide then
 x_offset is 320 (the left screen starts at "column" 320 of the 640).
*/

static int x_offset = 320;

void DisplaySetEye(int eye)
{
	x_offset = eye ? DisplayWidth : 0;
}

int DisplayInitialize()
{
	union REGS regs;
	int i;

	// allocate a screen buffer
	screen = malloc(64000);
	if (screen == NULL)
		return -1;

	/* compute the start of each row on the screen; the physical screen
		width is used, which is twice the DisplayWidth.
	*/
	for (i = 0; i < DisplayHeight; ++i)
		row_starts[i] = i * DisplayWidth * 2;  // 320 times 2 is 640

	// ask BIOS for current display mode
	regs.x.ax = 0x0F00;
	int86(0x10, &regs, &regs);
	oldmode = regs.h.al;         // save it away for later restoration

	// SET VIDEO MODE TO 320x400
	Set320x400Mode();
	return 0;
}



void DisplayUninitialize(void)
{
	// return to the original display mode
	union REGS regs;

	if (oldmode)
	{
		regs.x.ax = oldmode;
		int86(0x10, &regs, &regs);
		oldmode = 0;  // in case DisplayInitialize() is called again
	}
}



void setpixel(int X, int Y, int color)
{

	// Draws a pixel in the specified color at the specified
	// location in 320x400 256-color mode.
	// Input:
	// CX = X coordinate of pixel
	// DX = Y coordinate of pixel
	// BL = pixel color
	// Output: none
	// Registers altered: AX, CX, DX, DI, ES

	X = X + (x_offset);     // Breaks up lines for stereoscopic
	if ((Y % 2) == 0) X = X + 320;
	putX=X; putY=Y; putCOL=color;
	putp ();
}

/* This next routine is just Bresenham's; this particular piece of code
	is adapated from something written by Mark Feldman, from the PC-GPE
	(available via FTP on x2ftp.oulu.fi). */

void DisplayLine(int x1, int y1, int x2, int y2, int color)
{
	int deltax = abs(x2 - x1);
	int deltay = abs(y2 - y1);
	int i, numpixels, d, dinc1, dinc2;
	int x, xinc1, xinc2, y, yinc1, yinc2;

	if (deltax >= deltay)
	{
		numpixels = deltax;
		d = (deltay << 1) - deltax;
		dinc1 = deltay << 1;
		dinc2 = (deltay - deltax) << 1;
		xinc1 = 1;  xinc2 = 1;  yinc1 = 0;  yinc2 = 1;
	}
	else
	{
		numpixels = deltay;
		d = (deltax << 1) - deltay;
		dinc1 = deltax << 1;
		dinc2 = (deltax - deltay) << 1;
		xinc1 = 0;  xinc2 = 1;  yinc1 = 1;  yinc2 = 1;
	}
	if (x1 > x2)
	{
		xinc1 = -xinc1;
		xinc2 = -xinc2;
	}
	if (y1 > y2)
	{
		yinc1 = -yinc1;
		yinc2 = -yinc2;
	}
	x = x1;  y = y1;
	for (i = 0; i < numpixels; i++)
	{
		setpixel(x, y, color);
		if (d < 0)
		{
			d += dinc1;
			x += xinc1;
			y += yinc1;
		}
		else
		{
			d += dinc2;
			x += xinc2;
			y += yinc2;
		}
	}
}

