/* Demonstration of a spinning box (Stereoscopic)

 Written (320x200 mode) by Bernie Roehl, November 1994 for VictorMaxx Technologies
 Updated (to 320x400 mode) by Freddie Feldman, VictorMaxx Technologies, Inc., August 1995
 Redistribute at will, but charge nothing and give credit where due

 Make sure to include these files in your C Project File:
				VECTMAT.C
				RENDER.C
				MATH.C
				DISPLAY.C

 Redistribute at will, but charge nothing and give credit where due

 The options are as follows:

	 -e sets the eye spacing (interpupilary distance); default is 60 mm
	 -z sets the zoom factor; default is 1.00
	 -x sets the additional x-shift of the images; default is 0
	 -i sets the angle by which to increment the box's rotation each cycle;
		  the angle is in tenths of a degree, and the default is 100 (10
		  degrees)

	For example, the command:  sample -e75 -z1.5 -x10 -i50
	would set the eye spacing to 75 millimeters,    the zoom factor to 1.5,
	the x-shift to 10 pixels, and would increment the box's rotation around
	the vertical axis by 5.0 degrees every cycle.

	The following keys are active while the demo is running:

			ESC    quit the demo
			 p     pause the box's rotation (press any key to restart it)
			 [     decrease the x-shift value
			 ]     increase the x-shift value
			 +     zoom in
			 -     zoom out
			 =     reset zoom to 1.0
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include "3d.h"                         // Required for CyberMaxx Developers' Functions

Vector box_corners[] =
	{
	{ -75, -75, -250 },  { -75,  75, -250 },
	{  75,  75, -250 },  {  75, -75, -250 },
	{ -75, -75,  250 },  { -75,  75,  250 },
	{  75,  75,  250 },  {  75, -75,  250 }
	};

Edge box_edges[] =
	{
	{ 0, 1, 12 }, { 1, 2, 13 }, { 2, 3, 9 }, { 3, 0, 11 },   /* front */
	{ 4, 5, 13 }, { 5, 6, 12 }, { 6, 7, 11 }, { 7, 4, 10 },  /* back */
	{ 0, 4, 15 }, { 1, 5, 11 }, { 2, 6, 10 }, { 3, 7, 14 }   /* sides */
	};

Object box =
	{
	{ 0 },            /* location */
	{ 0 },            /* orientation */
	8, box_corners,   /* nvertices, vertices */
	12, box_edges     /* nedges, edges */
	};

void main(int argc, char *argv[])
	{
	int box_angle = 0;         // box's angle around the vertical (Y) axis
	int box_increment = 100;   // rotate the box by 10 degrees every cycle
	float eye_separation = 60; // assume 60 millimeters IPD
	int x_shift = 0;           // additional image shift
	int running = 1;           // this gets set to zero to exit
	float zoom_factor = 1.00;  // zoom factor
	char q;                                                 // used for pause
	// look for command-line parameters
	while (argc > 1)
		{
		if (argv[1][0] == '-')
			switch (argv[1][1])
				{
				case 'e': eye_separation = atof(&argv[1][2]); break;
				case 'i': box_increment = atoi(&argv[1][2]); break;
				case 'z': zoom_factor = atof(&argv[1][2]); break;
				case 'x': x_shift = atoi(&argv[1][2]); break;
				default: break;
				}
		--argc;
		++argv;
		}

	if (DisplayInitialize())
	{
		printf("Could not enter graphics mode!\n");
		exit(2);
	}

	RenderSetZoom(zoom_factor);

	BuildTrigTables();

	// position the first box in front of the user, and reset its rotation
	box.location.z = 1000;
	MatrixIdentity(box.orientation);

	while (running)
	{
		Matrix xmat, ymat, zmat, eye_matrix, tmp_mat;

		/* Rotate the box */

		Matrix tmp1, tmp2;
		box_angle += box_increment;   // rotate the box
		if (box_angle >= 1800)
			box_angle = -1800;  // keep it in range
		MatrixMakeYrot(tmp1, box_angle);
		MatrixMakeZrot(tmp2, 225);  // 22.5 degrees
		// Rotate the box around Z, then Y
		MatrixMultiply(box.orientation, tmp1, tmp2);

		// Create a transformation matrix for each rotation
		MatrixMakeXrot(xmat, 0);
		MatrixMakeYrot(ymat, 0);
		MatrixMakeZrot(zmat, 0);

		// Now multiply the matrices in the correct order
		MatrixMultiply(tmp_mat, xmat, ymat);  // rot around Y, then X
		MatrixMultiply(eye_matrix, zmat, tmp_mat);  //then Z

		/* The resulting eye_matrix will later be used to rotate an object's
			world coordinates into the eye coordinate frame (i.e. the frame
			in which the user is looking into the positive Z direction) */

		/* Actually do the rendering, left eye then right eye: */

		DisplayClear();    // clear the screen to color 0

		DisplaySetEye(0);   // left eye
		Render(&box, eye_matrix, -eye_separation/2, -x_shift);

		DisplaySetEye(1);   // right eye
		Render(&box, eye_matrix, eye_separation/2, x_shift);

		delay(25);
		// check if the user has pressed a key
		if (kbhit())
			{
			switch (getch())      // read the key
				{
				case 0x1B:                      // ESC key
					running = 0;                    // user wants out
					break;
				case 'p':                       // the 'p' key
					q=getch();                              // pauses box rotation
					break;
				case '[':                       // the '[' key
					--x_shift;                      // decreases shift
					break;
				case ']':                       // the ']' key
					++x_shift;                      // increases shift
					break;
				case '-':                       // the '-' key
					zoom_factor *= 0.9;     // decreases zoom
					RenderSetZoom(zoom_factor);
					break;
				case '+':                       // the '+' key
					zoom_factor *= 1.1;     // decreases zoom
					RenderSetZoom(zoom_factor);
					break;
				case '=':                       // the '+' key
					zoom_factor = 1.00;     // decreases zoom
					RenderSetZoom(zoom_factor);
					break;
				default:                        // any other key
					break;
				}
			}
		}
	DisplayUninitialize();
	}
